<?php

namespace Drupal\user_api_passwordless\Event;

use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Contracts\EventDispatcher\Event;

/**
 * Base class for passwordless login events.
 */
abstract class PasswordlessLoginEventBase extends Event {

  /**
   * If event is aborted.
   */
  protected bool $aborted = FALSE;

  /**
   * Error code.
   */
  protected string $errorCode = '';

  /**
   * Error message.
   */
  protected string $message = '';

  /**
   * HTTP status code.
   */
  protected int $statusCode = Response::HTTP_BAD_REQUEST;

  /**
   * Checks if event has been aborted.
   */
  public function isAborted(): bool {
    return $this->aborted;
  }

  /**
   * Abort the event and set error code and message.
   */
  public function abort(string $errorCode, string $message, ?int $statusCode) {
    $this->aborted = TRUE;
    $this->errorCode = $errorCode;
    $this->message = $message;

    if ($statusCode) {
      $this->statusCode = $statusCode;
    }
  }

  /**
   * Get error response.
   */
  public function getResponse(): Response {
    return new JsonResponse([
      'code' => $this->errorCode,
      'error' => $this->message,
    ], $this->statusCode);
  }

}
